@@ -36,12 +36,17 @@
 
 typedef struct
 {
-    CPUGraph  *base;
-    GtkWidget *color_buttons[NUM_COLORS];
-    GtkWidget *color_mode_combobox;
-    GtkBox    *hbox_highlight_smt, *hbox_in_terminal, *hbox_startup_notification;
-    GtkLabel  *smt_stats;
-    guint     timeout_id;
+    CPUGraph        *base;
+    GtkWidget       *color_buttons[NUM_COLORS];
+    GtkWidget       *color_mode_combobox;
+    GtkBox          *hbox_highlight_smt;
+    GtkBox          *hbox_in_terminal;
+    GtkBox          *hbox_per_core_spacing;
+    GtkBox          *hbox_startup_notification;
+    GtkToggleButton *per_core, *show_bars_checkbox;
+    GtkLabel        *smt_stats;
+    gchar           *smt_stats_tooltip;
+    guint           timeout_id;
 } CPUGraphOptions;
 
 static GtkBox *create_tab                    (void);
@@ -56,7 +61,8 @@ static GtkBox* create_check_box              (GtkBox          *tab,
                                               const gchar     *name,
                                               gboolean        init,
                                               void            (callback)(GtkToggleButton*, CPUGraphOptions*),
-                                              CPUGraphOptions *cb_data);
+                                              CPUGraphOptions *cb_data,
+                                              GtkToggleButton **out_checkbox);
 static GtkWidget* create_drop_down           (GtkBox          *tab,
                                               GtkSizeGroup    *sg,
                                               const gchar     *name,
@@ -97,6 +103,9 @@ static void    setup_color_mode_option       (GtkBox          *vbox,
 static void    setup_load_threshold_option   (GtkBox          *vbox,
                                               GtkSizeGroup    *sg,
                                               CPUGraph        *base);
+static GtkBox* setup_per_core_spacing_option (GtkBox          *vbox,
+                                              GtkSizeGroup    *sg,
+                                              CPUGraph        *base);
 
 static void    change_in_terminal            (GtkToggleButton *button,
                                               CPUGraphOptions *data);
@@ -130,7 +139,11 @@ static void    change_border                 (GtkToggleButton *button,
                                               CPUGraphOptions *data);
 static void    change_bars                   (GtkToggleButton *button,
                                               CPUGraphOptions *data);
-static void    change_size                   (GtkSpinButton   *sb,
+static void    change_per_core               (GtkToggleButton *button,
+                                              CPUGraphOptions *data);
+static void    change_per_core_spacing       (GtkSpinButton   *button,
+                                              CPUGraph        *base);
+static void    change_size                   (GtkSpinButton   *button,
                                               CPUGraph        *base);
 static void    change_smt                    (GtkToggleButton *button,
                                               CPUGraphOptions *data);
@@ -140,7 +153,7 @@ static void    change_update                 (GtkComboBox     *om,
                                               CPUGraphOptions *data);
 static void    change_core                   (GtkComboBox     *combo,
                                               CPUGraphOptions *data);
-static void    change_load_threshold         (GtkSpinButton   *sb,
+static void    change_load_threshold         (GtkSpinButton   *button,
                                               CPUGraph        *base);
 static gboolean update_cb                    (CPUGraphOptions *data);
 
@@ -152,7 +165,7 @@ create_options (XfcePanelPlugin *plugin, CPUGraph *base)
     GtkWidget *label, *notebook;
     GtkSizeGroup *sg;
     CPUGraphOptions *dlg_data;
-    gchar *smt_issues_tooltip, *smt_stats_tooltip;
+    gchar *smt_issues_tooltip;
 
     xfce_panel_plugin_block_menu (plugin);
 
@@ -178,25 +191,32 @@ create_options (XfcePanelPlugin *plugin, CPUGraph *base)
     setup_tracked_core_option (vbox, sg, dlg_data);
     setup_size_option (vbox, sg, plugin, base);
     setup_load_threshold_option (vbox, sg, base);
-    create_check_box (vbox, sg, _("Use non-linear time-scale"), base->non_linear, change_time_scale, dlg_data);
 
     gtk_box_pack_start (vbox, gtk_separator_new (GTK_ORIENTATION_HORIZONTAL), FALSE, FALSE, BORDER/2);
     setup_command_option (vbox, sg, dlg_data);
     dlg_data->hbox_in_terminal = create_check_box (vbox, sg, _("Run in terminal"),
-                                                   base->in_terminal, change_in_terminal, dlg_data);
+                                                   base->command_in_terminal, change_in_terminal, dlg_data,
+                                                   NULL);
     dlg_data->hbox_startup_notification = create_check_box (vbox, sg, _("Use startup notification"),
-                                                            base->startup_notification, change_startup_notification, dlg_data);
+                                                            base->command_startup_notification, change_startup_notification, dlg_data,
+                                                            NULL);
 
     smt_issues_tooltip = _("Color used to highlight potentially suboptimal\nplacement of threads on CPUs with SMT");
-    smt_stats_tooltip = g_strdup_printf("%s\n%s",
+    dlg_data->smt_stats_tooltip = g_strdup_printf("%s\n%s",
         _("'Overall' is showing the impact on the overall performance of the machine."),
         _("'Hotspots' is showing the momentary performance impact on just the threads involved in suboptimal SMT scheduling decisions."));
 
     gtk_box_pack_start (vbox, gtk_separator_new (GTK_ORIENTATION_HORIZONTAL), FALSE, FALSE, BORDER/2);
     dlg_data->hbox_highlight_smt = create_check_box (vbox, sg, _("Highlight suboptimal SMT scheduling"),
-                                                     base->highlight_smt, change_smt, dlg_data);
+                                                     base->highlight_smt, change_smt, dlg_data,
+                                                     NULL);
     setup_color_option (vbox, sg, dlg_data, SMT_ISSUES_COLOR, _("SMT issues color:"), smt_issues_tooltip, change_color_5);
 
+    gtk_box_pack_start (vbox, gtk_separator_new (GTK_ORIENTATION_HORIZONTAL), FALSE, FALSE, BORDER/2);
+    create_check_box (vbox, sg, _("Use non-linear time-scale"), base->non_linear, change_time_scale, dlg_data, NULL);
+    create_check_box (vbox, sg, _("Per-core history graphs"), base->per_core, change_per_core, dlg_data, &dlg_data->per_core);
+    dlg_data->hbox_per_core_spacing  = setup_per_core_spacing_option (vbox, sg, base);
+
     vbox2 = create_tab ();
     setup_color_option (vbox2, sg, dlg_data, FG_COLOR1, _("Color 1:"), NULL, change_color_1);
     setup_color_option (vbox2, sg, dlg_data, FG_COLOR2, _("Color 2:"), NULL, change_color_2);
@@ -206,15 +226,15 @@ create_options (XfcePanelPlugin *plugin, CPUGraph *base)
     setup_color_mode_option (vbox2, sg, dlg_data);
     gtk_box_pack_start (vbox2, gtk_separator_new (GTK_ORIENTATION_HORIZONTAL), FALSE, FALSE, BORDER/2);
     create_check_box (vbox2, sg, ngettext ("Show current usage bar", "Show current usage bars", base->nr_cores),
-                      base->has_bars, change_bars, dlg_data);
+                      base->has_bars, change_bars, dlg_data,
+                      &dlg_data->show_bars_checkbox);
     setup_color_option (vbox2, sg, dlg_data, BARS_COLOR, _("Bars color:"), NULL, change_color_4);
     gtk_box_pack_start (vbox2, gtk_separator_new (GTK_ORIENTATION_HORIZONTAL), FALSE, FALSE, BORDER/2);
-    create_check_box (vbox2, sg, _("Show frame"), base->has_frame, change_frame, dlg_data);
-    create_check_box (vbox2, sg, _("Show border"), base->has_border, change_border, dlg_data);
+    create_check_box (vbox2, sg, _("Show frame"), base->has_frame, change_frame, dlg_data, NULL);
+    create_check_box (vbox2, sg, _("Show border"), base->has_border, change_border, dlg_data, NULL);
 
     vbox3 = create_tab ();
     dlg_data->smt_stats = create_label_line (vbox3, "");
-    gtk_widget_set_tooltip_text (GTK_WIDGET (dlg_data->smt_stats), smt_stats_tooltip);
 
     notebook = gtk_notebook_new ();
     gtk_container_set_border_width (GTK_CONTAINER (notebook), BORDER - 2);
@@ -233,8 +253,6 @@ create_options (XfcePanelPlugin *plugin, CPUGraph *base)
 
     update_sensitivity (dlg_data);
     gtk_widget_show_all (dlg);
-
-    g_free (smt_stats_tooltip);
 }
 
 static GtkBox *
@@ -293,7 +311,8 @@ create_option_line (GtkBox *tab, GtkSizeGroup *sg, const gchar *name, const gcha
 
 static GtkBox*
 create_check_box (GtkBox *tab, GtkSizeGroup *sg, const gchar *name, gboolean init,
-                  void (callback)(GtkToggleButton*, CPUGraphOptions*), CPUGraphOptions *cb_data)
+                  void (callback)(GtkToggleButton*, CPUGraphOptions*), CPUGraphOptions *cb_data,
+                  GtkToggleButton **out_checkbox)
 {
     GtkBox *hbox;
     GtkWidget *checkbox;
@@ -305,6 +324,9 @@ create_check_box (GtkBox *tab, GtkSizeGroup *sg, const gchar *name, gboolean ini
     gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET (checkbox), FALSE, FALSE, 0);
     g_signal_connect (checkbox, "toggled", G_CALLBACK (callback), cb_data);
 
+    if (out_checkbox)
+        *out_checkbox = GTK_TOGGLE_BUTTON (checkbox);
+
     return hbox;
 }
 
@@ -337,6 +359,7 @@ destroy_cb (GtkWidget *dlg, CPUGraphOptions *data)
 {
     if (data->timeout_id)
         g_source_remove (data->timeout_id);
+    g_free (data->smt_stats_tooltip);
     g_free (data);
 }
 
@@ -364,7 +387,7 @@ setup_tracked_core_option (GtkBox *vbox, GtkSizeGroup *sg, CPUGraphOptions *data
 
     items[0] = _("All");
     for (i = 1; i < nb_items; i++)
-        items[i] = g_strdup_printf ("%u", i);
+        items[i] = g_strdup_printf ("%u", i-1);
 
     create_drop_down (vbox, sg, _("Tracked Core:"), (const gchar **) items, nb_items,
                       data->base->tracked_core, change_core, data);
@@ -384,7 +407,7 @@ setup_size_option (GtkBox *vbox, GtkSizeGroup *sg, XfcePanelPlugin *plugin, CPUG
     else
         hbox = create_option_line (vbox, sg, _("Height:"), NULL);
 
-    size = gtk_spin_button_new_with_range (10, 128, 1);
+    size = gtk_spin_button_new_with_range (MIN_SIZE, MAX_SIZE, 1);
     gtk_spin_button_set_value (GTK_SPIN_BUTTON (size), base->size);
     gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET (size), FALSE, FALSE, 0);
     g_signal_connect (size, "value-changed", G_CALLBACK (change_size), base);
@@ -403,6 +426,21 @@ setup_load_threshold_option (GtkBox *vbox, GtkSizeGroup *sg, CPUGraph *base)
     g_signal_connect (threshold, "value-changed", G_CALLBACK (change_load_threshold), base);
 }
 
+static GtkBox*
+setup_per_core_spacing_option (GtkBox *vbox, GtkSizeGroup *sg, CPUGraph *base)
+{
+    GtkBox *hbox;
+    GtkWidget *spacing;
+
+    hbox = create_option_line (vbox, sg, _("Spacing:"), NULL);
+    spacing = gtk_spin_button_new_with_range (PER_CORE_SPACING_MIN, PER_CORE_SPACING_MAX, 1);
+    gtk_spin_button_set_value (GTK_SPIN_BUTTON (spacing), base->per_core_spacing);
+    gtk_widget_set_tooltip_text (GTK_WIDGET (hbox), _("Spacing between per-core history graphs"));
+    gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET (spacing), FALSE, FALSE, 0);
+    g_signal_connect (spacing, "value-changed", G_CALLBACK (change_per_core_spacing), base);
+    return hbox;
+}
+
 static void
 setup_command_option (GtkBox *vbox, GtkSizeGroup *sg, CPUGraphOptions *data)
 {
@@ -557,11 +595,14 @@ update_sensitivity (const CPUGraphOptions *data)
 {
     const CPUGraph *base = data->base;
     const gboolean default_command = (base->command == NULL);
+    const gboolean per_core = base->nr_cores > 1 && base->tracked_core == 0 && base->mode != MODE_DISABLED;
 
     gtk_widget_set_sensitive (GTK_WIDGET (data->hbox_highlight_smt),
                               base->has_bars && base->topology && base->topology->smt);
     gtk_widget_set_sensitive (GTK_WIDGET (data->hbox_in_terminal), !default_command);
     gtk_widget_set_sensitive (GTK_WIDGET (data->hbox_startup_notification), !default_command);
+    gtk_widget_set_sensitive (GTK_WIDGET (data->per_core), per_core);
+    gtk_widget_set_sensitive (GTK_WIDGET (data->hbox_per_core_spacing), per_core && base->per_core);
 
     gtk_widget_set_sensitive (gtk_widget_get_parent (data->color_buttons[FG_COLOR2]),
                               base->color_mode != 0 || base->mode == MODE_LED || base->mode == MODE_GRID);
@@ -573,6 +614,7 @@ update_sensitivity (const CPUGraphOptions *data)
                               base->has_bars && base->highlight_smt && base->topology && base->topology->smt);
 
     gtk_widget_set_sensitive (gtk_widget_get_parent (data->color_mode_combobox), base->mode != MODE_GRID);
+    gtk_widget_set_sensitive (GTK_WIDGET (data->show_bars_checkbox), base->mode != MODE_DISABLED);
 }
 
 static void
@@ -597,6 +639,8 @@ change_mode (GtkComboBox *combo, CPUGraphOptions *data)
     }
 
     set_mode (data->base, mode);
+    if (mode == MODE_DISABLED && !data->base->has_bars)
+        gtk_toggle_button_set_active (data->show_bars_checkbox, TRUE);
     update_sensitivity (data);
 }
 
@@ -635,9 +679,22 @@ change_bars (GtkToggleButton *button, CPUGraphOptions *data)
 }
 
 static void
-change_size (GtkSpinButton *sb, CPUGraph *base)
+change_per_core (GtkToggleButton *button, CPUGraphOptions *data)
 {
-    set_size (base, gtk_spin_button_get_value_as_int (sb));
+    set_per_core (data->base, gtk_toggle_button_get_active (button));
+    update_sensitivity (data);
+}
+
+static void
+change_per_core_spacing (GtkSpinButton *button, CPUGraph *base)
+{
+    set_per_core_spacing (base, gtk_spin_button_get_value_as_int (button));
+}
+
+static void
+change_size (GtkSpinButton *button, CPUGraph *base)
+{
+    set_size (base, gtk_spin_button_get_value_as_int (button));
 }
 
 static void
@@ -648,9 +705,9 @@ change_smt (GtkToggleButton *button, CPUGraphOptions *data)
 }
 
 static void
-change_load_threshold (GtkSpinButton *sb, CPUGraph *base)
+change_load_threshold (GtkSpinButton *button, CPUGraph *base)
 {
-    set_load_threshold (base, gtk_spin_button_get_value (sb) / 100);
+    set_load_threshold (base, gtk_spin_button_get_value (button) / 100);
 }
 
 static void change_time_scale (GtkToggleButton *button, CPUGraphOptions *data)
@@ -666,6 +723,11 @@ static void change_update (GtkComboBox *combo, CPUGraphOptions *data)
 static void change_core (GtkComboBox *combo, CPUGraphOptions *data)
 {
     set_tracked_core (data->base, gtk_combo_box_get_active (combo));
+    if (data->base->tracked_core != 0)
+        set_per_core (data->base, FALSE);
+    else
+        set_per_core (data->base, gtk_toggle_button_get_active (data->per_core));
+    update_sensitivity (data);
 }
 
 static gboolean
@@ -673,6 +735,7 @@ update_cb (CPUGraphOptions *data)
 {
     const CPUGraph *base = data->base;
     gchar *smt_text;
+    gboolean show_tooltip = FALSE;
 
     if (base->topology)
     {
@@ -716,6 +779,8 @@ update_cb (CPUGraphOptions *data)
 
                 smt_text = g_strdup_printf ("%s\n%s\n%s\n\t%s\n\t%s", smt_detected,
                                             lines[0], lines[1], lines[2], lines[3]);
+
+                show_tooltip = TRUE;
             }
         }
         else
@@ -729,7 +794,10 @@ update_cb (CPUGraphOptions *data)
     }
 
     if (strcmp (gtk_label_get_text (data->smt_stats), smt_text))
+    {
         gtk_label_set_text (data->smt_stats, smt_text);
+        gtk_widget_set_tooltip_text (GTK_WIDGET (data->smt_stats), show_tooltip ? data->smt_stats_tooltip : "");
+    }
 
     g_free (smt_text);
 
