@@ -52,13 +52,15 @@ read_settings (XfcePanelPlugin *plugin, CPUGraph *base)
     XfceRc *rc;
 
     CPUGraphUpdateRate rate = RATE_NORMAL;
-    gboolean nonlinear = FALSE;
     CPUGraphMode mode = MODE_NORMAL;
     guint color_mode = 0;
-    gboolean frame = FALSE;
-    gboolean border = TRUE;
     gboolean bars = TRUE;
+    gboolean border = TRUE;
+    gboolean frame = FALSE;
     gboolean highlight_smt = HIGHLIGHT_SMT_BY_DEFAULT;
+    gboolean nonlinear = FALSE;
+    gboolean per_core = FALSE;
+    guint per_core_spacing = PER_CORE_SPACING_DEFAULT;
     guint tracked_core = 0;
 
     GdkRGBA colors[NUM_COLORS];
@@ -93,6 +95,8 @@ read_settings (XfcePanelPlugin *plugin, CPUGraph *base)
             border = xfce_rc_read_int_entry (rc, "Border", border);
             bars = xfce_rc_read_int_entry (rc, "Bars", bars);
             highlight_smt = xfce_rc_read_int_entry (rc, "SmtIssues", highlight_smt);
+            per_core = xfce_rc_read_int_entry (rc, "PerCore", per_core);
+            per_core_spacing = xfce_rc_read_int_entry (rc, "PerCoreSpacing", per_core_spacing);
             tracked_core = xfce_rc_read_int_entry (rc, "TrackedCore", tracked_core);
             load_threshold = xfce_rc_read_int_entry (rc, "LoadThreshold", load_threshold);
 
@@ -110,51 +114,59 @@ read_settings (XfcePanelPlugin *plugin, CPUGraph *base)
                 }
             }
 
-            switch (mode)
-            {
-                case MODE_DISABLED:
-                case MODE_NORMAL:
-                case MODE_LED:
-                case MODE_NO_HISTORY:
-                case MODE_GRID:
-                    break;
-                default:
-                    mode = MODE_NORMAL;
-            }
-
-            switch (rate)
-            {
-                case RATE_FASTEST:
-                case RATE_FAST:
-                case RATE_NORMAL:
-                case RATE_SLOW:
-                case RATE_SLOWEST:
-                    break;
-                default:
-                    rate = RATE_NORMAL;
-            }
-
             xfce_rc_close (rc);
         }
     }
 
-    set_update_rate (base, rate);
-    set_nonlinear_time (base, nonlinear);
-    set_size (base, size);
-    set_mode (base, mode);
+    // Validate settings
+    {
+        switch (mode)
+        {
+            case MODE_DISABLED:
+            case MODE_NORMAL:
+            case MODE_LED:
+            case MODE_NO_HISTORY:
+            case MODE_GRID:
+                break;
+            default:
+                mode = MODE_NORMAL;
+        }
+
+        if (mode == MODE_DISABLED && !bars)
+            mode = MODE_NORMAL;
+
+        switch (rate)
+        {
+            case RATE_FASTEST:
+            case RATE_FAST:
+            case RATE_NORMAL:
+            case RATE_SLOW:
+            case RATE_SLOWEST:
+                break;
+            default:
+                rate = RATE_NORMAL;
+        }
+    }
+
+    set_bars (base, bars);
+    set_border (base, border);
+    for (i = 0; i < NUM_COLORS; i++)
+        set_color (base, i, colors[i]);
     set_color_mode (base, color_mode);
-    set_frame (base, frame);
     if (command)
         set_command (base, command);
     set_in_terminal (base, in_terminal);
-    set_startup_notification (base, startup_notification);
-    set_border (base, border);
-    set_tracked_core (base, tracked_core);
-    set_bars (base, bars);
-    set_smt (base, highlight_smt);
-    for (i = 0; i < NUM_COLORS; i++)
-        set_color (base, i, colors[i]);
+    set_frame (base, frame);
     set_load_threshold (base, load_threshold * 0.01f);
+    set_mode (base, mode);
+    set_nonlinear_time (base, nonlinear);
+    set_per_core (base, per_core);
+    set_per_core_spacing (base, per_core_spacing);
+    set_size (base, size);
+    set_smt (base, highlight_smt);
+    set_startup_notification (base, startup_notification);
+    set_tracked_core (base, tracked_core);
+    set_update_rate (base, rate);
     g_free (command);
 }
 
@@ -175,19 +187,20 @@ write_settings (XfcePanelPlugin *plugin, CPUGraph *base)
         return;
 
     xfce_rc_write_int_entry (rc, "UpdateInterval", base->update_interval);
-    xfce_rc_write_int_entry (rc, "TimeScale", base->non_linear);
+    xfce_rc_write_int_entry (rc, "TimeScale", base->non_linear ? 1 : 0);
     xfce_rc_write_int_entry (rc, "Size", base->size);
     xfce_rc_write_int_entry (rc, "Mode", base->mode);
-    xfce_rc_write_int_entry (rc, "Frame", base->has_frame);
-    xfce_rc_write_int_entry (rc, "Border", base->has_border);
-    xfce_rc_write_int_entry (rc, "Bars", base->has_bars);
+    xfce_rc_write_int_entry (rc, "Frame", base->has_frame ? 1 : 0);
+    xfce_rc_write_int_entry (rc, "Border", base->has_border ? 1 : 0);
+    xfce_rc_write_int_entry (rc, "Bars", base->has_bars ? 1 : 0);
+    xfce_rc_write_int_entry (rc, "PerCore", base->per_core ? 1 : 0);
     xfce_rc_write_int_entry (rc, "TrackedCore", base->tracked_core);
     if (base->command)
         xfce_rc_write_entry (rc, "Command", base->command);
     else
         xfce_rc_delete_entry (rc, "Command", FALSE);
-    xfce_rc_write_int_entry (rc, "InTerminal", base->in_terminal);
-    xfce_rc_write_int_entry (rc, "StartupNotification", base->startup_notification);
+    xfce_rc_write_int_entry (rc, "InTerminal", base->command_in_terminal ? 1 : 0);
+    xfce_rc_write_int_entry (rc, "StartupNotification", base->command_startup_notification ? 1 : 0);
     xfce_rc_write_int_entry (rc, "ColorMode", base->color_mode);
     if (base->load_threshold != 0)
         xfce_rc_write_int_entry (rc, "LoadThreshold", (gint) roundf (100 * base->load_threshold));
@@ -217,9 +230,14 @@ write_settings (XfcePanelPlugin *plugin, CPUGraph *base)
     }
 
     if (base->highlight_smt != HIGHLIGHT_SMT_BY_DEFAULT)
-        xfce_rc_write_int_entry (rc, "SmtIssues", base->highlight_smt);
+        xfce_rc_write_int_entry (rc, "SmtIssues", base->highlight_smt ? 1 : 0);
     else
         xfce_rc_delete_entry (rc, "SmtIssues", FALSE);
 
+    if (base->per_core_spacing != PER_CORE_SPACING_DEFAULT)
+        xfce_rc_write_int_entry (rc, "PerCoreSpacing", base->per_core_spacing);
+    else
+        xfce_rc_delete_entry (rc, "PerCoreSpacing", FALSE);
+
     xfce_rc_close (rc);
 }
